package com.kuxuan.laraver.app;

import com.joanzapata.iconify.IconFontDescriptor;
import com.joanzapata.iconify.Iconify;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.WeakHashMap;

/**
 *
 * @author huangfuruixin
 * @date 2017/12/5
 */

public class Configurator {
    private static final HashMap<String,Object> LARAVER_CONFIGS = new HashMap<>();
    //字体图标库
    private static final ArrayList<IconFontDescriptor> ICONS = new ArrayList<>();

    private Configurator(){
        LARAVER_CONFIGS.put(ConfigType.CONFIG_READY.name(),false);

    }

    public static Configurator getInstance(){
        return Holder.INSTANCE;
    }

    final HashMap<String,Object> getLaraverConfigs(){
        return LARAVER_CONFIGS;
    }


    /**
     * 静态内部类实现线程安全的懒汉单例
     */
    private static class Holder {
        private static final Configurator INSTANCE = new Configurator();
    }

    public final void configure(){
        initIcons();
        LARAVER_CONFIGS.put(ConfigType.CONFIG_READY.name(),true);

    }

    /**
     * 配置API  HOST
     * @param host HOST
     * @return
     */
     public final Configurator withApiHost(String host){
       LARAVER_CONFIGS.put(ConfigType.API_HOST.name(),host);
       return this;
     }

    /**
     * 初始化字体图标
     */
    private  void initIcons(){
         if(ICONS.size()>0){
             final Iconify.IconifyInitializer initializer = Iconify.with(ICONS.get(0));
             for(int i=1;i<ICONS.size();i++){
                 initializer.with(ICONS.get(i));
             }
         }
     }

    /**
     * 假如自己的图标
     * @param descriptor
     * @return
     */
     public final Configurator withIcon(IconFontDescriptor descriptor){
        ICONS.add(descriptor);
        return this;
     }

    /**
     * 检查配置项是否完成
     */
    private void checkConfiguration(){
         final boolean isReady = (boolean) LARAVER_CONFIGS.get(ConfigType.CONFIG_READY.name());
         if(!isReady){
             throw new RuntimeException("Configuration is not ready,call configure");
         }
     }

    /**
     * 获取配置
     * @param key  key
     * @param <T>   t
     * @return
     */
     @SuppressWarnings("unchecked")
     final <T> T getConfiguration(Enum<ConfigType> key){
        checkConfiguration();
        return (T)LARAVER_CONFIGS.get(key.name());

     }

}
