package com.kuxuan.laraver.net;

import java.util.Map;

import okhttp3.MultipartBody;
import okhttp3.ResponseBody;
import retrofit2.Call;
import retrofit2.http.DELETE;
import retrofit2.http.FieldMap;
import retrofit2.http.FormUrlEncoded;
import retrofit2.http.GET;

import retrofit2.http.Multipart;
import retrofit2.http.POST;
import retrofit2.http.Part;
import retrofit2.http.QueryMap;
import retrofit2.http.Streaming;
import retrofit2.http.Url;

/**
 * @author huangfuruixin
 * @date 2017/12/5
 */

public interface RestService {
    /**
     * get请求
     *
     * @param url    url
     * @param params 请求参数
     * @return call
     */
    @GET
    Call<String> get(@Url String url, @QueryMap Map<String, Object> params);


    /**
     * post请求
     *
     * @param url    url
     * @param params 请求参数
     * @return call
     */
    @FormUrlEncoded
    @POST
    Call<String> post(@Url String url, @FieldMap Map<String, Object> params);


    /**
     * get请求
     *
     * @param url    url
     * @param params 请求参数
     * @return call
     */
    @FormUrlEncoded
    @POST
    Call<String> put(@Url String url, @FieldMap Map<String, Object> params);


    /**
     * delete请求
     *
     * @param url    url
     * @param params 请求参数
     * @return call
     */
    @DELETE
    Call<String> delete(@Url String url, @QueryMap Map<String, Object> params);


    /**
     * download请求
     * Stream注解 避免内存溢出，边下载边写入
     *
     * @param url    url
     * @param params 请求参数
     * @return call
     */
    @Streaming
    @GET
    Call<ResponseBody> download(@Url String url, @QueryMap Map<String, Object> params);


    /**
     * upload
     *
     * @param url  url
     * @param file file
     * @return Call
     */
    @Multipart
    @POST
    Call<String> upload(@Url String url, @Part MultipartBody.Part file);

}
